@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include "JackAtomic.h"
 #include "JackCompilerDeps.h"
 #include <string.h> // for memcpy
+#include <cstddef>
 
 namespace Jack
 {
@@ -93,7 +94,7 @@ class JackAtomicState
     protected:
 
         T fState[2];
-        volatile AtomicCounter fCounter;
+        alignas(UInt32) volatile AtomicCounter fCounter;
         SInt32 fCallWriteCounter;
 
         UInt32 WriteNextStateStartAux()
@@ -131,6 +132,8 @@ class JackAtomicState
 
         JackAtomicState()
         {
+            static_assert(offsetof(JackAtomicState, fCounter) % sizeof(fCounter) == 0,
+                          "fCounter must be aligned within JackAtomicState");
             Counter(fCounter) = 0;
             fCallWriteCounter = 0;
         }
