@@ -12,7 +12,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with this program; if not, write to the Free Software 
+along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 */
@@ -45,7 +45,10 @@ enum NotificationType {
     kRealTimeCallback = 14,
     kShutDownCallback = 15,
     kQUIT = 16,
-    kMaxNotification
+    kSessionCallback = 17,
+    kLatencyCallback = 18,
+    kPropertyChangeCallback = 19,
+    kMaxNotification = 64  // To keep some room in JackClientControl fCallback table
 };
 
 } // end of namespace
