@@ -1,20 +1,20 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 
@@ -29,9 +29,9 @@
 namespace Jack
 {
 
-void JackWinEvent::BuildName(const char* name, const char* server_name, char* res)
+void JackWinEvent::BuildName(const char* name, const char* server_name, char* res, int size)
 {
-    sprintf(res, "jack_pipe.%s_%s", server_name, name);
+    snprintf(res, size, "jack_pipe.%s_%s", server_name, name);
 }
 
 bool JackWinEvent::Signal()
@@ -89,7 +89,7 @@ bool JackWinEvent::TimedWait(long usec)
 // Client side : get the published semaphore from server
 bool JackWinEvent::ConnectInput(const char* name, const char* server_name)
 {
-    BuildName(name, server_name, fName);
+    BuildName(name, server_name, fName, sizeof(fName));
     jack_log("JackWinEvent::Connect %s", fName);
 
     // Temporary...
@@ -130,7 +130,7 @@ bool JackWinEvent::Disconnect()
 
 bool JackWinEvent::Allocate(const char* name, const char* server_name, int value)
 {
-    BuildName(name, server_name, fName);
+    BuildName(name, server_name, fName, sizeof(fName));
     jack_log("JackWinEvent::Allocate name = %s val = %ld", fName, value);
 
     /* create an auto reset event */
